
#include "hmatrixfile.h"
#include "hmatrix.h"
#include <TError.h>
#include <iostream>

using namespace std;

// ****************************************************************************
ClassImp(HMatrix)

const Double_t HMatrix::kDeg2Rad = TMath::Pi()/180.;

//-----------------------------------------------------------------------------
HMatrix::HMatrix(Int_t id, const char* matrname, const HMatrixFile& filename) : id(id), name(matrname), status(kFALSE)
{
   const TH1 *tmp = filename.get( matrname );
   if (0 != tmp)
   {
      hist = dynamic_cast<TH1*>( tmp->Clone() );
      size = hist->GetDimension();
      for (int i=0; i<3; ++i) 
      {
         mapping[i] = i;
         val[i] = 0.;
         valscale[i] = 1.;
         lower_limit[i] = 0.;
         upper_limit[i] = 0.;
      }

      lower_limit[0] = hist->GetXaxis()->GetXmin();
      upper_limit[0] = hist->GetXaxis()->GetXmax();
      if (size > 1)
      {
         lower_limit[1] = hist->GetYaxis()->GetXmin();
         upper_limit[1] = hist->GetYaxis()->GetXmax();
      }
      if (size > 2)
      {
         lower_limit[2] = hist->GetZaxis()->GetXmin();
         upper_limit[2] = hist->GetZaxis()->GetXmax();
      }
   
      // -------------------------------------------------------------------
      // here we get some predefined values for single Acc and Eff matrices
      // -------------------------------------------------------------------
      
      if (name.find("Acc") != string::npos)
      {
         type = "acc";
         setXaxis("mom", "MeV", kFALSE, kTRUE);
         setYaxis("theta","deg", kFALSE, kFALSE);
         setZaxis("phi","deg", kFALSE, kFALSE);
         status = kTRUE;
      }
      if (name.find("Eff") != string::npos)
      {
         if (size == 2) 
         {
            type = "eff";
            setXaxis("mom", "MeV", kFALSE, kTRUE);
            setYaxis("theta","deg", kFALSE, kFALSE);
            status = kTRUE;
         }
         if (size == 3)
         {
            type = "eff";
            setXaxis("phi", "rad", kFALSE, kFALSE);
            setYaxis("theta","rad", kFALSE, kFALSE);
            setZaxis("mom","GeV", kFALSE, kTRUE);
            status = kTRUE;
         }
      }
   }
   else
   {
      Error("HMatrix", "Histogram '%s' not found in opened files.", matrname);
      status = kFALSE;
   }
}


//-----------------------------------------------------------------------------
void HMatrix::setXaxis(const char* name, const char* unit, Bool_t lower, Bool_t upper)
{
   xname = name;
   xunit = unit;

   if (xname=="mom") mapping[0] = 0; 
    else if (xname=="theta") mapping[1] = 0;
    else if (xname=="phi")  mapping[2] = 0; 

   if (xunit == "GeV") valscale[0] = 0.001; 
    else if (xunit == "rad")  valscale[0] = kDeg2Rad;

   extrLower[0] = lower;
   extrUpper[0] = upper;
}
	
//-----------------------------------------------------------------------------
void HMatrix::setYaxis(const char* name, const char* unit, Bool_t lower, Bool_t upper)
{
   yname = name;
   yunit = unit;

   if (yname=="mom") mapping[0] = 1; 
    else if (yname=="theta") mapping[1] = 1;
    else if (yname=="phi")  mapping[2] = 1; 

   if (yunit == "GeV") valscale[1] = 0.001; 
    else if (yunit == "rad")  valscale[1] = kDeg2Rad;

   extrLower[1] = lower;
   extrUpper[1] = upper;
}
	
//-----------------------------------------------------------------------------
void HMatrix::setZaxis(const char* name, const char* unit, Bool_t lower, Bool_t upper)
{
   zname = name;
   zunit = unit;

   if (zname=="mom") mapping[0] = 2; 
    else if (zname=="theta") mapping[1] = 2;
    else if (zname=="phi")  mapping[2] = 2; 

   if (zunit == "GeV") valscale[2] = 0.001; 
    else if (zunit == "rad")  valscale[2] = kDeg2Rad;

   extrLower[2] = lower;
   extrUpper[2] = upper;
}


//-----------------------------------------------------------------------------
void HMatrix::setVal(Double_t x)
{
   val[mapping[0]] = x * valscale[mapping[0]];
}


//-----------------------------------------------------------------------------
void HMatrix::setVal(Double_t x, Double_t y)
{
   val[mapping[0]] = x * valscale[mapping[0]];
   val[mapping[1]] = y * valscale[mapping[1]];
}   

//-----------------------------------------------------------------------------
void HMatrix::setVal(Double_t x, Double_t y, Double_t z)
{
   val[mapping[0]] = x * valscale[mapping[0]];
   val[mapping[1]] = y * valscale[mapping[1]];
   val[mapping[2]] = z * valscale[mapping[2]];
}   



//-----------------------------------------------------------------------------
Double_t HMatrix::operator()(Double_t x) const
{
   const_cast<HMatrix*>(this) -> setVal(x);

   double shift = 0.0;
   if (val[mapping[0]] < lower_limit[mapping[0]])
   {
      if (extrLower[mapping[0]]) {
         shift += 0.0001; // I want the first bin value
      } else return 0.;
   }
   else if (val[mapping[0]] > upper_limit[mapping[0]])
   {
      if (extrUpper[mapping[0]]) {
         shift -= 0.0001; // I want the last bin value
      } else return 0.;
   }

 return hist->GetBinContent(hist->FindBin(val[0] + shift));
}



//-----------------------------------------------------------------------------
Double_t HMatrix::operator()(Double_t x, Double_t y) const
{
   const_cast<HMatrix*>(this) -> setVal(x, y);

   double shift = 0.0, shift2 = 0.0;
   if (val[mapping[0]] < lower_limit[mapping[0]])
   {
      if (extrLower[mapping[0]]) {
         shift += 0.0001; // I want the first bin value
      } else return 0.;
   }
   else if (val[mapping[0]] > upper_limit[mapping[0]])
   {
      if (extrUpper[mapping[0]]) {
         shift -= 0.0001; // I want the last bin value
      } else return 0.;
   }

   if (val[mapping[1]] < lower_limit[mapping[1]])
   {
      if (extrLower[mapping[1]]) {
         shift2 += 0.0001; // I want the first bin value
      } else return 0.;
   }
   else if (val[mapping[1]] > upper_limit[mapping[1]])
   {
      if (extrUpper[mapping[1]]) {
         shift2 -= 0.0001; // I want the last bin value
      } else return 0.;
   }

 return hist->GetBinContent( hist->FindBin(val[0] + shift, val[1] + shift2) );
}


//-----------------------------------------------------------------------------
Double_t HMatrix::operator()(Double_t x, Double_t y, Double_t z) const
{
   const_cast<HMatrix*>(this) -> setVal(x, y, z);

   double shift = 0.0, shift2 = 0.0, shift3 = 0.0;;
   if (val[mapping[0]] < lower_limit[mapping[0]])
   {
      if (extrLower[mapping[0]]) {
         shift += 0.0001; // I want the first bin value
      } else return 0.;
   }
   else if (val[mapping[0]] > upper_limit[mapping[0]])
   {
      if (extrUpper[mapping[0]]) {
         shift -= 0.0001; // I want the last bin value
      } else return 0.;
   }

   if (val[mapping[1]] < lower_limit[mapping[1]])
   {
      if (extrLower[mapping[1]]) {
         shift2 += 0.0001; // I want the first bin value
      } else return 0.;
   }
   else if (val[mapping[1]] > upper_limit[mapping[1]])
   {
      if (extrUpper[mapping[1]]) {
         shift2 -= 0.0001; // I want the last bin value
      } else return 0.;
   }

   if (val[mapping[2]] < lower_limit[mapping[2]])
   {
      if (extrLower[mapping[2]]) {
         shift3 += 0.0001; // I want the first bin value
      } else return 0.;
   }
   else if (val[mapping[2]] > upper_limit[mapping[2]])
   {
      if (extrUpper[mapping[2]]) {
         shift3 -= 0.0001; // I want the last bin value
      } else return 0.;
   }
 return hist->GetBinContent( hist->FindBin(val[0] + shift, val[1] + shift2, val[2] + shift3) );
}




